
*
* WinSplit.gms:	The basic winvest model, solved as a 3-stage stochastic
*		program, using the split-variable formulation with
*		non-anticipativity constraints.
*		Can use 4-scenario data from WinData.inc, or the scenario
*		generation code from WinGen.inc, by changing the include:
*
* SN, GAMS Summer Schools, 2003.
*
$include "WinData.inc"


* Define the model:
  positive variables 
       buy(t, i, s),
       sell(t, i, s),
       hold(t, i, s),
       cash_hold(t,s);

  variables wealth(s), 
            objval;

  equations
       mbsflow(t, i, s),
       cashflow(t, s),
       wealthdef(s),
       nonant1(i, s),
       nonant2(i, s1, s),
       linobj;

* Flow conservation for securities:

  mbsflow(t, i, s) ..
         buy(t, i, s)                          $ (ord(t) lt card(t))
         + (yield(i, t-1, s)* hold(t-1, i, s)) $ (ord(t) gt 1)
    =e=
         sell(t, i, s)                         $ (ord(t) gt 1)
         +  hold(t, i, s)                      $ (ord(t) lt card(t));

* Flow conservation for cash:

  cashflow(t, s) ..
        sum(i,  sell(t, i, s) * (1-transcost)) $ (ord(t) gt 1)
      + (cash_yield(t-1, s)*cash_hold(t-1, s)) $ (ord(t) gt 1)
      + 100                                    $ (ord(t) eq 1)
   =e=
         sum(i, buy(t, i, s))                  $ (ord(t) lt card(t))
      +  cash_hold(t, s)                       
      +  liab(t, s)                            $ liabs;

* The final wealth under each scenario is the cash remaining at the end,
*  after the final value of the liabilities has been subtracted:

  wealthdef(s) ..
         wealth(s) =e= cash_hold("t2", s) - (val(s)$liabs);

  linobj  ..   objval =e= sum(s, wealth(s)) / card(s); 

* Add first- and second-stage nonanticipativity conditions:
  nonant1(i, s) $ (ord(s) lt card(s)) ..
                   hold("t0", i, s) =e= hold("t0", i, s+1);

* The second-stage non-anticipativity constraints bind all
* scenarios to their "representative" scenario at time t1:
  nonant2(i, repr(s1,s)) .. hold("t1", i, s1) =E= hold("t1", i, s);

  sell.up("t0",i,s) = 0;

  model WinSplit /mbsflow, cashflow, wealthdef,linobj, nonant1, nonant2/;


* Model 3: Solve stochastic model in split-variable formulation:
  solve WinSplit maximizing objval using lp;
  display buy.l, sell.l, wealth.l;


