
*
* winmodel.gms:	The basic winvest model. Can use either the standard
*		4-scenario data from windata.inc, or the scenario
*		generation code from wingen.inc, by changing the
*		include:
*
$include "wingen.inc"


* Define the model:
  positive variables 
       buy(t, i, s),
       sell(t, i, s),
       hold(t, i, s),
       cash_hold(t,s);

  variables wealth(s), 
            objval;

  equations
       mbsflow(t, i, s),
       cashflow(t, s),
       wealthdef(s),
       nonant1(i, s),
       nonant2(i, s),
       linobj;

* Flow conservation for securities:

  mbsflow(t, i, s) ..
         buy(t, i, s)                          $ (ord(t) lt card(t))
         + (yield(i, t-1, s)* hold(t-1, i, s)) $ (ord(t) gt 1)
    =e=
         sell(t, i, s)                         $ (ord(t) gt 1)
         +  hold(t, i, s)                      $ (ord(t) lt card(t));

* Flow conservation for cash:

  cashflow(t, s) ..
        sum(i,  sell(t, i, s) * (1-transcost)) $ (ord(t) gt 1)
      + (cash_yield(t-1, s)*cash_hold(t-1, s)) $ (ord(t) gt 1)
      + 100                                    $ (ord(t) eq 1)
   =e=
         sum(i, buy(t, i, s))                  $ (ord(t) lt card(t))
      +  cash_hold(t, s)                       
      +  liab(t, s)                            $ liabs;

* The final wealth under each scenario is the cash remaining at the end,
*  after the final value of the liabilities has been subtracted:

  wealthdef(s) ..
         wealth(s) =e= cash_hold("t2", s) - (val(s)$liabs);

  linobj  ..   objval =e= sum(s, wealth(s)) / card(s); 

* Add first- and second-stage nonanticipativity conditions:
  nonant1(i, s) $ (ord(s) lt card(s)) ..
                   hold("t0", i, s) =e= hold("t0", i, s+1);
  nonant2(i, s) $ (ord(s) eq 1 or ord(s) eq 3) ..
                   hold("t1", i, s) =e= hold("t1", i, s+1);

  sell.up("t0",i,s) = 0;

  model independnt /mbsflow, cashflow, wealthdef,linobj/;
  model split /mbsflow, cashflow, wealthdef,linobj, nonant1, nonant2/;


* Model 1: Solve without transaction costs, indep. per scenario, no liabilities:
  transcost = 0.00;
  liabs = 0;
  solve independnt maximizing objval using lp;
  display buy.l, sell.l, wealth.l;

* Model 2: Solve with 1% transaction cost, indep. per scenario:
  transcost = 0.01;
  solve independnt maximizing objval using lp;
  display buy.l, sell.l, wealth.l;

* Model 3: Solve stochastic model in split-variable formulation:
  solve split maximizing objval using lp;
  display buy.l, sell.l, wealth.l;

* Now move into the nonlinear model, max. expected utility:

  equation utildef;
  utildef ..  objval =e= sum(s, log(wealth(s)+0.0001) ) / card(s);

  model util /mbsflow, cashflow, wealthdef,utildef, nonant1, nonant2/;

  solve util maximizing objval using nlp;
  display buy.l, sell.l, wealth.l;

  parameter ce, er;
  ce = exp(objval.l); er = sum(s, wealth.l(s)) / card(s);
  display ce,er;

