*  BA380M Case 2: Winvest. 
*  1/6/93: Updated cash.t1 figures.

 option decimals=6, limcol = 0, limrow = 0;

 set Cscen /uu, ud, dd, du/;
 set Cmbs  /cash, io2, po7, po70, io90/;
 set Ctime /t0, t1, t2/;

table yield(Cmbs, Ctime, Cscen)
                        UU          UD          DD          DU
       CASH.T0    1.030414    1.030414    1.012735    1.012735
       CASH.T1    1.032623    1.014298    1.009788    1.030481
       IO2 .T0    1.104439    1.104439    0.959238    0.959238
       IO2 .T1    1.110009    0.975907    0.935106    1.167817
       PO7 .T0    0.938159    0.938159    1.166825    1.166825
       PO7 .T1    0.933668    1.154590    1.156536    0.903233
       PO70.T0    0.924840    0.924840    1.167546    1.167546
       PO70.T1    0.891527    1.200802    1.141917    0.907837
       IO90.T0    1.107461    1.107461    0.908728    0.908728
       IO90.T1    1.105168    0.925925    0.877669    1.187143 ;


 table liab(Ctime, Cscen)
                         UU          UD          DD          DU
             T1   26.474340   26.474340   10.953843   10.953843
             T2   31.264791   26.044541   10.757200   13.608207 ;

 parameter val(Cscen)
   / uu = 47.284751, ud = 49.094838, dd = 86.111238, du = 83.290085/;

 parameter transcost; transcost = 0.01
 parameter liabs;     liabs = 0;

 display yield, liab;

* Define the model:
  positive variables 
       buy(Ctime, Cmbs, Cscen),
       sell(Ctime, Cmbs, Cscen),
       hold(Ctime, Cmbs, Cscen);

  variables wealth(Cscen), 
            objval;

  equations
       mbsflow(Ctime, Cmbs, Cscen),
       cashflow(Ctime, Cscen),
       wealthdef(Cscen),
       nonant(Cmbs, Cscen),
       linobj;

* Flow conservation for securities (excluding cash):

  mbsflow(Ctime, Cmbs, Cscen) $ (ord(Cmbs) ne 1) ..
         buy(Ctime, Cmbs, Cscen)
      +  (yield(Cmbs, Ctime-1, Cscen)* hold(Ctime-1, Cmbs, Cscen))
                       $ (ord(Ctime) gt 1)
    =e=
         sell(CTime, Cmbs, Cscen)
      +  hold(Ctime, Cmbs, Cscen);

* Flow conservation for cash:

  cashflow(Ctime, Cscen) ..
         sum(Cmbs$(ord(Cmbs) ne 1),  
                   sell(Ctime, Cmbs, Cscen) * (1-transcost))
      +  (yield("cash",Ctime-1, Cscen)*hold(Ctime-1, "cash", Cscen))
                       $ (ord(Ctime) gt 1)
      +  100 $ (ord(Ctime) eq 1)
   =e=
         sum(Cmbs$(ord(Cmbs) ne 1),  
                   buy(Ctime, Cmbs, Cscen))
      +  hold(Ctime, "cash", Cscen)
      +  liab(Ctime, Cscen) $ liabs;

* The final wealth under each scenario is the cash remaining at the end,
*  after the final value of the liabilities has been subtracted:

  wealthdef(Cscen) ..
         wealth(Cscen) =e= hold("t2", "cash", Cscen) - (val(Cscen)$liabs);

  linobj  ..   objval =e= sum(Cscen, wealth(Cscen)) / card(Cscen); 

* nonant(Cmbs, Cscen) $ (mod(ord(Cscen),2) eq 1) .. 
*                  buy("t0", Cmbs, Cscen) =e= buy("t0", Cmbs, Cscen+1);
  nonant(Cmbs, Cscen) $ (ord(Cscen) lt card(Cscen)) ..
                   buy("t0", Cmbs, Cscen) =e= buy("t0", Cmbs, Cscen+1);
  sell.up("t0",Cmbs,Cscen) = 0;

  model investment /mbsflow, cashflow, wealthdef,linobj/;
  model stoch /mbsflow, cashflow, wealthdef,linobj, nonant/;


* Model 1: Solve without transaction costs, indep. per scenario, no liabilities:
  transcost = 0.00;
  liabs = 0;
  solve investment maximizing objval using lp;
  display buy.l, sell.l, wealth.l;

* Model 2: Solve with 1% transaction cost, indep. per scenario:
  transcost = 0.01;
  solve investment maximizing objval using lp;
  display buy.l, sell.l, wealth.l;

* Model 3: Solve stochastic model:
  solve stoch maximizing objval using lp;
  display buy.l, sell.l, wealth.l;

* Model 4: Solve stochastic model WITH liabilities:
  liabs = 1;
  solve stoch maximizing objval using lp;
  display buy.l, sell.l, wealth.l;

* Model 5: Add the constraints that final returns are non-neg. under each scen.
  
  wealth.lo(Cscen) = 0.001;
  solve stoch maximizing objval using lp;
  display buy.l, sell.l, wealth.l;
  

* Now move into the nonlinear model, max. expected utility:

  parameter alpha;

  equation utildef;
  utildef ..  objval =e= sum(Cscen, 
             (1/(1-alpha)*(wealth(Cscen)**(1-alpha)-1))$(alpha ne 1)
           +  log(wealth(Cscen))$(alpha eq 1)
                                   ) / card(Cscen);

  model util /mbsflow, cashflow, wealthdef,utildef, nonant/;

* liabs = 0;
* solve util maximizing objval using nlp;
* display buy.l, sell.l, wealth.l;

  liabs = 1; alpha = 0;
  solve util maximizing objval using nlp;
  display buy.l, sell.l, wealth.l;

  liabs = 1; alpha = 0.5;
  solve util maximizing objval using nlp;
  display buy.l, sell.l, wealth.l;

  liabs = 1; alpha = 1;
  solve util maximizing objval using nlp;
  display buy.l, sell.l, wealth.l;

  liabs = 1; alpha = 2;
  solve util maximizing objval using nlp;
  display buy.l, sell.l, wealth.l;

  liabs = 1; alpha = 10;
  solve util maximizing objval using nlp;
  display buy.l, sell.l, wealth.l;

  parameter ce, er;
  ce = exp(objval.l); er = sum(Cscen, wealth.l(Cscen)) / card(Cscen);
  display ce,er;

