
*  MSC 380 Case 2: Winvest. S. Nielsen.
*    1/6/93: Updated cash.t1 figures.
*  11/11/96: Separated cash out from securities.
*  11/25/96: Corrected bug.

 option decimals=6, limcol = 0, limrow = 0, solprint = off;

 set Cscen /uu, ud, dd, du/;              alias(s, Cscen);
 set Cmbs  /io2, po7, po70, io90/;  alias(i, Cmbs);
 set Ctime /t0, t1, t2/;                  alias(t, Ctime);

table yield(i,t,s)
                        UU          UD          DD          DU
       IO2 .T0    1.104439    1.104439    0.959238    0.959238
       IO2 .T1    1.110009    0.975907    0.935106    1.167817
       PO7 .T0    0.938159    0.938159    1.166825    1.166825
       PO7 .T1    0.933668    1.154590    1.156536    0.903233
       PO70.T0    0.924840    0.924840    1.167546    1.167546
       PO70.T1    0.891527    1.200802    1.141917    0.907837
       IO90.T0    1.107461    1.107461    0.908728    0.908728
       IO90.T1    1.105168    0.925925    0.877669    1.187143 ;

table cash_yield(t, s)

                        UU          UD          DD          DU
            T0    1.030414    1.030414    1.012735    1.012735
            T1    1.032623    1.014298    1.009788    1.030481 ;

 table liab(t,s)
                        UU          UD          DD          DU
            T1   26.474340   26.474340   10.953843   10.953843
            T2   31.264791   26.044541   10.757200   13.608207 ;

 parameter val(s)
   / uu = 47.284751, ud = 49.094838, dd = 86.111238, du = 83.290085/;

 parameter transcost; transcost = 0.01
 parameter liabs;     liabs = 0;

 display yield, liab;

* Define the model:
  positive variables 
       buy(t, i, s),
       sell(t, i, s),
       hold(t, i, s),
       cash_hold(t,s);

  variables wealth(s), 
            objval;

  equations
       mbsflow(t, i, s),
       cashflow(t, s),
       wealthdef(s),
       nonant1(i, s),
       nonant2(i, s),
       linobj;

* Flow conservation for securities:

  mbsflow(t, i, s) ..
         buy(t, i, s)                          $ (ord(t) lt card(t))
         + (yield(i, t-1, s)* hold(t-1, i, s)) $ (ord(t) gt 1)
    =e=
         sell(t, i, s)                         $ (ord(t) gt 1)
         +  hold(t, i, s)                      $ (ord(t) lt card(t));

* Flow conservation for cash:

  cashflow(t, s) ..
        sum(i,  sell(t, i, s) * (1-transcost)) $ (ord(t) gt 1)
      + (cash_yield(t-1, s)*cash_hold(t-1, s)) $ (ord(t) gt 1)
      + 100                                    $ (ord(t) eq 1)
   =e=
         sum(i, buy(t, i, s))                  $ (ord(t) lt card(t))
      +  cash_hold(t, s)                       
      +  liab(t, s)                            $ liabs;

* The final wealth under each scenario is the cash remaining at the end,
*  after the final value of the liabilities has been subtracted:

  wealthdef(s) ..
         wealth(s) =e= cash_hold("t2", s) - (val(s)$liabs);

  linobj  ..   objval =e= sum(s, wealth(s)) / card(s); 

* Add first- and second-stage nonanticipativity conditions:
  nonant1(i, s) $ (ord(s) lt card(s)) ..
                   hold("t0", i, s) =e= hold("t0", i, s+1);
  nonant2(i, s) $ (ord(s) eq 1 or ord(s) eq 3) ..
                   hold("t1", i, s) =e= hold("t1", i, s+1);

  sell.up("t0",i,s) = 0;

  model independnt /mbsflow, cashflow, wealthdef,linobj/;
  model stoch /mbsflow, cashflow, wealthdef,linobj, nonant1, nonant2/;


* Model 1: Solve without transaction costs, indep. per scenario, no liabilities:
  transcost = 0.00;
  liabs = 0;
  solve independnt maximizing objval using lp;
  display buy.l, sell.l, wealth.l;

* Model 2: Solve with 1% transaction cost, indep. per scenario:
  transcost = 0.01;
  solve independnt maximizing objval using lp;
  display buy.l, sell.l, wealth.l;

* Model 3: Solve stochastic model:
  solve stoch maximizing objval using lp;
  display buy.l, sell.l, wealth.l;

* Model 4: Solve stochastic model WITH liabilities:
  liabs = 1;
  solve stoch maximizing objval using lp;
  display buy.l, sell.l, wealth.l;

* Model 5: Add the constraints that final returns are non-neg. under each scen.
  
  wealth.lo(s) = 0;
  solve stoch maximizing objval using lp;
  display buy.l, sell.l, wealth.l;
  

* Now move into the nonlinear model, max. expected utility:

  equation utildef;
  utildef ..  objval =e= sum(s, log(wealth(s)+0.0001) ) / card(s);

  model util /mbsflow, cashflow, wealthdef,utildef, nonant1, nonant2/;

* liabs = 0;
* solve util maximizing objval using nlp;
* display buy.l, sell.l, wealth.l;

  liabs = 0;
  solve util maximizing objval using nlp;
  display buy.l, sell.l, wealth.l;

  parameter ce, er;
  ce = exp(objval.l); er = sum(s, wealth.l(s)) / card(s);
  display ce,er;

